unit BitmapPlus;
{$P+,S-,W-,R-,T-,X+,H+}
{$C PRELOAD}

//----------------------------------------------------------
interface

uses  SysUtils, Classes, Graphics, Windows;

//----------------------------------------------------------
const
        mtNone=0;
        mtLzw=1;
//----------------------------------------------------------
type
  TBitmapPlus = class(Graphics.TBitmap)
private
  FQuality     : integer;
  FMethod      : integer;
  procedure     SetQuality      ( value: integer);
  procedure     SetMethod       ( value: integer);

public
  destructor    Destroy; override;
  constructor   Create; override;
  procedure     LoadFromFile(const FileName: string);overload;//override;
  procedure     LoadFromFile(const FileName: string; Frame: integer);overload;
  procedure     SaveToFile(const FileName: string); override;

  property      Quality		 : integer           read FQuality   write SetQuality;  // 0...100
  property      Method		 : integer           read FMethod    write SetMethod;   // mtNone, mtLzw
end;
//----------------------------------------------------------

implementation
uses  Math,  GDIPAPI, GDIPOBJ, GDIPUTIL;

//----------------------------------------------------------
type
  LogPal = packed record
  lpal : TLogPalette;
  Colors: Array[0..255] of TPaletteEntry;
  end;
//----------------------------------------------------------
type
  ColorPaletteP = packed record
  Flags : UINT ;
  Count : UINT ;
  Entries: array [0..255] of ARGB ;
  end;
//----------------------------------------------------------
function IsClsid(const encname: string; const Clsid: TGUID): bool;
var guid: TGUID;
begin
	GetEncoderClsid(encname, guid);
	Result:=(Clsid.D1=guid.D1);
end;
//----------------------------------------------------------
function EncoderClsid(const FileName: string; var guid: TGUID): Boolean;
var encname, ext: string;
begin
	Result:=false;
	ext  :=UpperCase(ExtractFileExt(FileName));
	if ext='.PNG' then encname:='image/png'
	else if ext='.JPG' then encname:='image/jpeg'
	else if ext='.JPEG' then encname:='image/jpeg'
	else if ext='.TIFF' then encname:='image/tiff'
	else if ext='.TIF' then encname:='image/tiff'
	else if ext='.GIF' then encname:='image/gif'
	else encname:='image/bmp';
	if encname<>'' then Result:=(GetEncoderClsid(encname, guid)>-1);
end;
//----------------------------------------------------------
var  parameterValue    : EncoderValue;
procedure GetSaveParameters(const Clsid: TGUID; var Parameters: EncoderParameters; Quality: integer; Method: integer);
begin
	Parameters.Count:=1;
	Parameters.Parameter[0].Type_ := EncoderParameterValueTypeLong;
	Parameters.Parameter[0].NumberOfValues := 1;
	Parameters.Parameter[0].Value := @parameterValue;
	if IsClsid('image/jpeg', Clsid) then
        	begin
	        	Parameters.Parameter[0].Guid := EncoderQuality;
		        parameterValue := EncoderValue(Quality);
        	end
	else if IsClsid('image/tiff', Clsid) then
	        begin
		        Parameters.Count:=1;
        		Parameters.Parameter[0].Guid := EncoderCompression;
	        	case (Method) of
		        	1: parameterValue := EncoderValueCompressionLZW;
//			        2: parameterValue := EncoderValueCompressionRle;                // isn't supported by GDI+ now
//      			3: parameterValue := EncoderValueCompressionCCITT3;
//	        		4: parameterValue := EncoderValueCompressionCCITT4;
		        	else parameterValue := EncoderValueCompressionNone;
        		end;
	        end
	else 	Parameters.Count := 0;
end;

//----------------------------------------------------------
constructor TBitmapPlus.Create;
begin
	 inherited Create;
	 FQuality:=50;
	 FMethod:=mtNone;
end;
//----------------------------------------------------------
destructor TBitmapPlus.Destroy;
begin
   inherited Destroy;
end;
//----------------------------------------------------------
procedure TBitmapPlus.LoadFromFile(const FileName: string);
begin
        LoadFromFile(FileName,0);
end;
//----------------------------------------------------------
procedure TBitmapPlus.LoadFromFile(const FileName: string; Frame: integer);
var
        Image           : TGPBitmap;
	GraphicsP       : TGPGraphics;
        pageGuid        : TGUID;
	fGuid           : TGUID;
        SysPal          : LogPal;
        size            : integer;
        PaletteP        : ColorPaletteP;
        i               : integer;
begin
	Image:=TGPBitmap.Create(FileName);
	Image.GetRawFormat(fGuid);
	if (fGuid.D1=ImageFormatGIF.D1)  then  pageGuid :=FrameDimensionTime
	else     pageGuid := FrameDimensionPage;
	Image.SelectActiveFrame(pageGuid, Frame);
	Width:=Image.GetWidth;
	Height:=Image.GetHeight;
        case (Image.GetPixelFormat) of
        	 PixelFormat1bppIndexed: PixelFormat:=pf1bit;
	         PixelFormat4bppIndexed: PixelFormat:=pf4bit;
        	 PixelFormat8bppIndexed: PixelFormat:=pf8bit;
	         PixelFormat16bppARGB1555: PixelFormat:=pf16bit;
        	 PixelFormat16bppGrayScale: PixelFormat:=pf16bit;
	         PixelFormat16bppRGB555: PixelFormat:=pf16bit;
        	 PixelFormat16bppRGB565: PixelFormat:=pf16bit;
	         PixelFormat24bppRGB:    PixelFormat:=pf24bit;
        	 PixelFormat32bppARGB:   PixelFormat:=pf32bit;
	         PixelFormat32bppPARGB:  PixelFormat:=pf32bit;
        	 PixelFormat32bppRGB:    PixelFormat:=pf32bit;
        	 else                    PixelFormat:=pf24bit;
	end;
        if (PixelFormat=pf1bit) or (PixelFormat=pf4bit) or (PixelFormat=pf8bit) then
        begin
             size:=Image.GetPaletteSize;
             Image.GetPalette(PColorPalette(@PaletteP), size);
             size:=min(256, (size div 4) -2);
             SysPal.lPal.palVersion:=$300;
             SysPal.lPal.palNumEntries:=size;
             for i:=0 to size-1 do      SysPal.Colors[i]:=TPaletteEntry(ARGBToColorRef(PaletteP.Entries[i]));
             Palette := CreatePalette(Syspal.lpal);
        end;
	GraphicsP  := TGPGraphics.Create(Canvas.Handle);
	GraphicsP.DrawImage(Image, 0,0,Image.GetWidth,Image.GetHeight);
	GraphicsP.Free;
	Image.Free();
	if(@Canvas.OnChange<>nil) then Canvas.OnChange(Canvas);
end;
//----------------------------------------------------------
procedure TBitmapPlus.SaveToFile(const FileName: string);
var
	 Clsid      : TGUID;
	 Parameters : EncoderParameters;
	 Img        : TGPBitmap;
begin
	Img:=TGPBitmap.Create(Handle,Palette);
	if (Img<>nil)and EncoderClsid(FileName, Clsid) then
	begin
		  DeleteFile(PChar(FileName));
		  GetSaveParameters(Clsid, Parameters, Quality, Method);
		  Img.Save(FileName, Clsid, @Parameters);
	end;
	Img.Free;
end;
//----------------------------------------------------------
procedure TBitmapPlus.SetQuality( value: integer);
begin
	FQuality:=value;
end;
//----------------------------------------------------------
procedure TBitmapPlus.SetMethod( value: integer);
begin
	FMethod:=value;
end;
//----------------------------------------------------------
end.


